from celery.result import AsyncResult


__all__ = [
    'AsyncDetailedResult',
]


class AsyncDetailedResult(AsyncResult):
    def __init__(self, id, backend=None,
                 task_name=None,            # deprecated
                 app=None, parent=None):
        super().__init__(id, backend, task_name, app, parent)

    def details(self):
        meta = self._get_task_meta()
        return meta.get('details') if meta else []



# encoding: utf-8

from __future__ import unicode_literals
import logging
import random
import six
from base64 import b64decode
from base64 import b64encode
from Crypto.Cipher import AES
from Crypto.Random import get_random_bytes
from struct import pack
from django.conf import settings


__all__ = [
    u'encrypt',
    u'decrypt',
]


logger = logging.getLogger(__name__)


def encrypt(plain_text, key=None):
    """
    The encrypt function encrypts a unicode string using the
    Blowfish cipher (provided by pycrypto).  The key used is
    the SECRET_KEY specified in the settings file.

    @param plain_text The plaintext unicode string to be encrypted.

    @return The encrypted ciphertext.

    """

    block_size = AES.block_size
    if not isinstance(plain_text, six.string_types):
        plain_text = '%s' % plain_text
    b_plain_text = b64encode(plain_text.encode('utf-8'))
    iv = get_random_bytes(block_size)
    e_cipher = AES.new(
        key or settings.DJENGA_ENCRYPTION_KEY, AES.MODE_CBC, iv=iv)
    padding_len = block_size - len(b_plain_text) % block_size
    padding = [padding_len] * padding_len
    padding = pack('b' * padding_len, *padding)
    cipher_text = e_cipher.iv + e_cipher.encrypt(b_plain_text + padding)
    return cipher_text


def decrypt(cipher_text, key=None):
    """
    The decrypt function decrypts unicode strings that have
    been encrypted by the util.encryption.encrypt() function.
    The cipher used is Blowfish (provided by pcrypto), and the
    key used is the SECRET_KEY specified in the settings file.

    @param cipher_text The encrypted ciphertext.

    @return The decrypted plaintext (unicode) string.

    >>> import os
    >>> os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'settings')
    >>> from django.conf import settings
    >>> settings.configure()
    >>> settings.DJENGA_ENCRYPTION_KEY = 'sesame'
    >>> st = 'hello'
    >>> decrypt(encrypt(st)) == st
    True
    """

    block_size = AES.block_size
    d_cipher = AES.new(key or settings.DJENGA_ENCRYPTION_KEY, AES.MODE_CBC)
    d_b64text = d_cipher.decrypt(cipher_text)
    plain_text = b64decode(d_b64text[block_size:]).decode('utf-8')
    return plain_text


if __name__ == "__main__":
    plain_text = 'Hello sesame!'
    key = b'0123456789012345'
    print(decrypt(encrypt(plain_text, key), key))





from __future__ import absolute_import
from collections import OrderedDict
import logging
from time import time
from celery.app.task import Task, Context
from celery import states


__all__ = [
    'DetailTask',
    'TaskDetail',
]
logger = logging.getLogger(__name__)


class TaskDetail:
    def __init__(self, key=None, description=None, *args, **kwargs):
        """
        :type key: str
        :type description: str
        """
        self.key = key
        self.description = description
        self.detail = []
        self.millis = None
        self.done = False
        self.error = False

    def add_detail(self, detail):
        self.detail.append(detail)

    def start_time(self):
        self.millis = time()

    def end_time(self):
        self.done = True
        self.millis = int(1000 * (time() - (self.millis or 0)))

    def to_json(self):
        return {
            'key': self.key,
            'description': self.description,
            'details': self.detail,
            'latest': self.detail[-1] if self.detail else None,
            'time': self.millis,
            'done': self.done,
        }

    def __str__(self):
        return '%s: %s' % (self.key, self.description,)


class DetailTask(Task):
    steps = None

    def __init__(self, steps=None):
        super().__init__()
        self.steps = self.steps or []
        self.details = OrderedDict()
        for key, description in self.steps:
            self.details[key] = description

    def initialize_detail(self):
        if not hasattr(self.request, 'details'):
            self.request.details = OrderedDict([
                (key, TaskDetail(key, description),)
                for key, description in self.details.items()
            ])
            """:type dict[str, TaskDetail]"""
            self.request.detail_stack = list()
            self.request.current_detail = None

    def save_details(self):
        self.backend.store_result(
            self.request.id,
            result=None,
            state=states.STARTED,
            details=self.request.details)

    def start_step(self, key, description=None, detail='in progress'):
        self.initialize_detail()
        r = self.request
        if key not in r.details:
            r.current_detail = r.details.setdefault(
                key, TaskDetail(key, description))
        else:
            r.current_detail = r.details[key]
        r.current_detail.start_time()
        r.current_detail.add_detail(detail)
        r.detail_stack.append(r.current_detail)
        logger.info(
            '[%s/%s] %s', r.current_detail.key,
            r.current_detail.description, detail)
        self.save_details()

    def update_step(self, detail, *args):
        self.initialize_detail()
        r = self.request
        if args:
            try:
                detail = detail % args
            except Exception as ex:
                logger.exception('%s', ex)
        r.current_detail.add_detail(detail)
        logger.info(
            '[%s/%s] %s', r.current_detail.key,
            r.current_detail.description, detail)
        self.save_details()

    def end_step(self, error=None, detail='done'):
        self.initialize_detail()
        r = self.request
        if error:
            r.current_detail.error = error
        r.current_detail.end_time()
        r.current_detail.add_detail(detail)
        logger.info(
            '[%s/%s] %s', r.current_detail.key,
            r.current_detail.description, detail)
        r.detail_stack.pop()
        if r.detail_stack:
            r.current_detail = r.detail_stack[-1]
        else:
            r.current_detail = None
        self.save_details()



from __future__ import absolute_import, unicode_literals
import os
from celery import Celery
from celery.signals import after_setup_logger
from celery.signals import worker_process_init
from djenga.celery.tasks import DetailTask


# set the default Django settings module for the 'celery' program.
os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'djenga_tests.settings')


class DjengaCelery(Celery):
    def __init__(self, main=None, loader=None, backend=None,
                 amqp=None, events=None, log=None, control=None,
                 set_as_current=True, tasks=None, broker=None, include=None,
                 changes=None, config_source=None, fixups=None, task_cls=None,
                 autofinalize=True, namespace=None, strict_typing=True,
                 **kwargs):
        from djenga.celery.tasks import DetailTask
        from djenga.celery.backends import patch_aliases
        patch_aliases()
        super().__init__(main, loader, backend, amqp, events, log, control,
                         set_as_current, tasks, broker, include, changes,
                         config_source, fixups, task_cls=DetailTask,
                         autofinalize=autofinalize, namespace=namespace,
                         strict_typing=strict_typing, **kwargs)

    def now(self):
        """Return the current time and date as a datetime."""
        from datetime import datetime
        return datetime.now(self.timezone)


app = DjengaCelery('djenga_tests')


# Using a string here means the worker don't have to serialize
# the configuration object to child processes.
# - namespace='CELERY' means all celery-related configuration keys
#   should have a `CELERY_` prefix.
app.config_from_object('django.conf:settings', namespace='CELERY')
# Load task modules from all registered Django app configs.
app.autodiscover_tasks()
app.conf.broker_transport_options = {'visibility_timeout': 86400}
n = 0


@app.task(bind=True, steps=[
    ( 1, 'get logger',),
    ( 2, 'increment n',),
    ( 3, 'output debugging',), ]
)
def debug_task(self):
    """:type self: djenga.celery.tasks.DetailTask"""
    import logging
    import time
    self.start_step(1)
    logger = logging.getLogger(__name__)
    time.sleep(30)
    self.end_step()
    self.start_step(2)
    global n
    n += 1
    time.sleep(45)
    self.end_step()
    self.start_step(3)
    logger.info('request type: %s', type(self.request))
    print('{0} Request: {1!r}'.format(n, self.request))
    time.sleep(60)
    self.end_step()
    logger.info('details: ')
    for x in self.request.details.values():
        logger.info('  - %s [%s]', x, x.done)


@app.task(bind=True, base=DetailTask)
def error_task(self):
    import logging
    logger = logging.getLogger(__name__)
    global n
    n += 1
    logger.info('request type: %s', type(self.request))
    logger.error('{0} Request: {1!r}'.format(n, self.request))


def update_loglevel(*args, **kwargs):
    app.log.redirect_stdouts(loglevel='INFO')


# it's not at all clear to me why these
# two signals work, or the correct timing at
# which to call the function to redirect the
# stdouts, but this worked, so I felt it
# was wise to just go with it . . .
after_setup_logger.connect(update_loglevel)
worker_process_init.connect(update_loglevel)

from djenga.celery.utils import auto_step


@auto_step(key=1)
def fly_to_the_moon(self):
    pass


@auto_step(key=2)
def shrink_the_moon(self):
    pass


@auto_step(key=3)
def grab_the_moon(self):
    pass


@auto_step(key=4)
def sit_on_the_toilet(self):
    pass


@app.task(bind=True, base=DetailTask, steps=[
            (1, 'Fly to the moon'), (2, 'Shrink the moon'),
            (3, 'Grab the moon'), (4, 'Sit on the toilet'),])
def steal_the_moon(self):
    fly_to_the_moon(self)
    shrink_the_moon(self)
    grab_the_moon(self)
    sit_on_the_toilet(self)
